function getNodesInRange(range) {
    var start = range.startContainer;
    var end = range.endContainer;
    var commonAncestor = range.commonAncestorContainer;
    var nodes = [];
    var node;

    // walk parent nodes from start to common ancestor
    for (node = start.parentNode; node; node = node.parentNode) {
        //modified to only add text nodes to the array
        if (node.nodeType == 3 && node.nodeValue.length > 0) {
            nodes.push(node);
        }
        if (node == commonAncestor) {
            break;
        }
    }
    nodes.reverse();

    // walk children and siblings from start until end is found
    for (node = start; node; node = getNextNode(node)) {
        //modified to only add text nodes to the array
        if (node.nodeType == 3 && node.nodeValue.length > 0) {
            nodes.push(node);
        }
        if (node == end) {
            break;
        }
    }
    return nodes;
}

function getNextNode(node) {
    if (node.firstChild) {
        return node.firstChild;
    }
    while (node) {
        if (node.nextSibling) {
            return node.nextSibling;
        }
        node = node.parentNode;
    }
}

function applyClassToRange(range, cssClass, color, id) {
    var startRange, endRange;
    if (range.collapsed) {
        startRange = range.cloneRange();
        endRange = range.cloneRange();
    } else {
        startRange = range.cloneRange();
        startRange.collapse(true);
        endRange = range.cloneRange();
        endRange.collapse(false);
    }

    var startOffset = 0;
    var startNode = startRange.startContainer;
    if (startNode.nodeType == 3) {
        startOffset = startRange.startOffset;
    } else {
        var textNode = document.createTextNode("");
        startRange.insertNode(textNode);
        startNode = textNode;
        startRange.setStart(textNode, 0);
        startRange.setEnd(textNode, 0);
    }

    var endOffset = 0;
    var endNode = endRange.endContainer;
    if (endNode.nodeType == 3) {
        endOffset = endRange.endOffset;
    } else {
        var textNode = document.createTextNode("");
        endRange.insertNode(textNode);
        endNode = textNode;
    }
    range.setStart(startNode, startOffset);
    range.setEnd(endNode, endOffset);

    // Get the start and end nodes and split them to give new start and
    // end nodes with only text that falls inside the range.
    startNode = range.startContainer.splitText(range.startOffset);
    endNode = range.endContainer.splitText(range.endOffset).previousSibling;

    // Adjust the range to contain the new start and end nodes
    // The offsets are not really important anymore but might as well set
    // them correctly
    range.setStart(startNode, 0);
    range.setEnd(endNode, endNode.length);

    // Get an array of all text nodes within the range
    var nodes = getNodesInRange(range);

    // Place span tags with style around each textnode
    for (i = 0; i < nodes.length; ++i) {
        var span = document.createElement("span");
        span.setAttribute("class", cssClass);
        span.setAttribute("id", id)
        span.style.backgroundColor = color + ' !important';
        span.appendChild(document.createTextNode(nodes[i].nodeValue));
        nodes[i].parentNode.replaceChild(span, nodes[i]);
    }
}