/*
 * Copyright (C) 2012 Brandon Tate
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* Namespace*/
var android = {};
android.selection = {};

android.selection.selectionStartRange = null;
android.selection.selectionEndRange = null;


/** The last point touched by the user. { 'x': xPoint, 'y': yPoint } */
android.selection.lastTouchPoint = null;


/**
 * Starts the touch and saves the given x and y coordinates as last touch point
 */
android.selection.startTouch = function (x, y) {
    /*console.log("startTouch");*/
    android.selection.lastTouchPoint = {'x': parseInt(x), 'y': parseInt(y)};
    /*console.log("x:" + x + " y:" + y);*/
    /*console.log("parsed x:" + parseInt(x) + " parsed y:" + parseInt(y));*/
};


/**
 *    Checks to see if there is a selection.
 *
 *    @return boolean
 */
android.selection.hasSelection = function () {
    return window.getSelection().toString().length > 0;
};


/**
 *    Clears the current selection.
 */
android.selection.clearSelection = function () {

    try {
        /* if current selection clear it.*/
        var sel = window.getSelection();
        sel.removeAllRanges();
    } catch (err) {
        window.TextSelection.jsError("clearSelection: " + err);
    }
};


/**
 *    Handles the long touch action by selecting the last touched element.
 */
android.selection.longTouch = function () {

    /*try{*/

    android.selection.clearSelection();

    /* if current selection clear it.*/
    var sel = window.getSelection();

    if (document.caretRangeFromPoint) {
        var oneWordCaret = document.caretRangeFromPoint(android.selection.lastTouchPoint.x, android.selection.lastTouchPoint.y);
        oneWordCaret.expand("word");
    }
    else {
        /*console.log('caretRangeFromPoint doesn\'t exist');*/
    }


    sel.addRange(oneWordCaret);

    var temp = sel.getRangeAt(0);

    android.selection.saveSelectionStart();
    android.selection.saveSelectionEnd();


    android.selection.selectionChanged();


    /* }
     catch(err){
     window.TextSelection.jsError("LongTouch: "+err);
     }*/

};

/**
 * Tells the app to show the context menu.
 */
android.selection.selectionChanged = function () {

    /*try{*/

    var sel = window.getSelection();
    if (!sel) {
        return;
    }

    var range = sel.getRangeAt(0);
    var firstCharHeight = android.selection.getFirstCharHeight(range);
    var lastCharHeight = android.selection.getLastCharHeight(range);

    /* Add spans to the selection to get page offsets*/
    var selEnd = document.createElement('span');
    selEnd.id = 'selectionEnd';
    var selectionEnd = $(selEnd);//$("<span id=\"selectionEnd\"/>");

    var selStart = document.createElement('span');
    selStart.id = 'selectionStart'
    var selectionStart = $(selStart);//$("<span id=\"selectionStart\"/>");
    //selectionStart.id="selectionStart";


    var startRange = document.createRange();
    startRange.setStart(range.startContainer, range.startOffset);
    startRange.insertNode(selectionStart[0]);

    var endRange = document.createRange();
    endRange.setStart(range.endContainer, range.endOffset);
    endRange.insertNode(selectionEnd[0]);


    /* Create the bounds json object for the selection*/
    var handleBounds = "{'left': " + selectionStart.offset().left + ", ";
    handleBounds += "'top': " + (selectionStart.offset().top + firstCharHeight) + ", ";
    handleBounds += "'right': " + selectionEnd.offset().left + ", ";
    handleBounds += "'bottom': " + (selectionEnd.offset().top + lastCharHeight) + "}";


    /* Pull the spans*/
    selectionStart.remove();
    selectionEnd.remove();

    /* Reset range */
    sel.removeAllRanges();
    sel.addRange(range);

    /* Menu bounds*/
    var rect = range.getBoundingClientRect();
    var startChar = range2Index(startRange);
    var endChar = range2Index(endRange);
    /*console.log("start: " + startChar + "; endChar: " + endChar);*/

    var menuBounds = "{'left': " + rect.left + ", ";
    menuBounds += "'top': " + rect.top + ", ";
    menuBounds += "'right': " + rect.right + ", ";
    menuBounds += "'bottom': " + rect.bottom + "}";

    /* Rangy*/
    var rangyRange = android.selection.getRange();

    /* Text to send to the selection*/
    var text = window.getSelection().toString();

    /* Set the content width*/
    window.TextSelection.setContentWidth(document.body.clientWidth);

    /* Tell the interface that the selection changed*/
    window.TextSelection.selectionChanged(rangyRange, text, handleBounds, menuBounds, startChar, endChar);
    /*}
     catch(err){
     window.TextSelection.jsError("selectionChanged: "+err);
     }*/
};


android.selection.getRange = function () {
    var serializedRangeSelected = rangy.serializeSelection();
    var serializerModule = rangy.modules.Serializer;
    if (serializedRangeSelected != '') {
        if (rangy.supported && serializerModule && serializerModule.supported) {
            var beginingCurly = serializedRangeSelected.indexOf("{");
            serializedRangeSelected = serializedRangeSelected.substring(0, beginingCurly);
            return serializedRangeSelected;
        }
    }
}


/**
 * Returns the last touch point as a readable string.
 */
android.selection.lastTouchPointString = function () {
    if (android.selection.lastTouchPoint == null)
        return "undefined";

    return "{" + android.selection.lastTouchPoint.x + "," + android.selection.lastTouchPoint.y + "}";
};


android.selection.saveSelectionStart = function () {
    try {

        /* Save the starting point of the selection*/
        var sel = window.getSelection();
        var range = sel.getRangeAt(0);

        var saveRange = document.createRange();

        saveRange.setStart(range.startContainer, range.startOffset);

        android.selection.selectionStartRange = saveRange;
    } catch (err) {
        window.TextSelection.jsError("saveSelectionStart: " + err);
    }

};

android.selection.saveSelectionEnd = function () {

    try {

        /* Save the end point of the selection*/
        var sel = window.getSelection();
        var range = sel.getRangeAt(0);

        var saveRange = document.createRange();
        saveRange.setStart(range.endContainer, range.endOffset);

        android.selection.selectionEndRange = saveRange;
    } catch (err) {
        window.TextSelection.jsError("saveSelectionEnd: " + err);
    }

};


/**
 *    returns the height of the first character of given range. (used for positioning the selection handles)
 */
android.selection.getFirstCharHeight = function (range) {
    try {
        var startRange = range.cloneRange();
        startRange.setEnd(range.startContainer, range.startOffset + 1);
        var rect = startRange.getBoundingClientRect();
        var height = rect.bottom - rect.top;
        /*console.log("FirstCharHeight: height " + height + ";     ");*/
        return height;
    } catch (err) {
        window.TextSelection.jsError("getFirstCharHeight: " + err);
    }
}

/**
 *    returns the height of the last character of given range. (used for positioning the selection handles)
 */
android.selection.getLastCharHeight = function (range) {
    try {
        var endRange = range.cloneRange();
        endRange.setStart(range.endContainer, range.endOffset - 1);
        var rect = endRange.getBoundingClientRect();
        var height = rect.bottom - rect.top;
        /*console.log("LastCharHeight: height " + height + ";     ");*/
        return height;
    } catch (err) {
        window.TextSelection.jsError("getLastCharHeight: " + err);
    }
}


/**
 * Sets the last caret position for the start handle.
 */
android.selection.setStartPos = function (x, y) {

    try {
        android.selection.selectionStartRange = document.caretRangeFromPoint(parseFloat(x + ""), parseFloat(y + ""));

        android.selection.selectBetweenHandles();
    } catch (err) {
        window.TextSelection.jsError("setStartPos " + err);
    }

};

/**
 * Sets the last caret position for the end handle.
 */
android.selection.setEndPos = function (x, y) {

    try {
        android.selection.selectionEndRange = document.caretRangeFromPoint(parseFloat(x + ""), parseFloat(y + ""));

        android.selection.selectBetweenHandles();

    } catch (err) {
        window.TextSelection.jsError("setEndPos " + err);
    }

};

/**
 *    Selects all content between the two handles
 */
android.selection.selectBetweenHandles = function () {

    try {
        var startCaret = android.selection.selectionStartRange;
        var endCaret = android.selection.selectionEndRange;

        /* If we have two carets, update the selection*/
        if (startCaret && endCaret) {

            /* If end caret comes before start caret, need to flip*/
            if (startCaret.compareBoundaryPoints(Range.START_TO_END, endCaret) > 0) {
                var temp = startCaret;
                startCaret = endCaret;
                endCaret = temp;

                android.selection.selectionStartRange = startCaret;
                android.selection.selectionEndRange = endCaret;
            }

            var range = document.createRange();
            range.setStart(startCaret.startContainer, startCaret.startOffset);
            range.setEnd(endCaret.startContainer, endCaret.startOffset);


            android.selection.clearSelection();

            var selection = window.getSelection();
            selection.addRange(range);


        }

        android.selection.selectionChanged();
    }
    catch (err) {
        window.TextSelection.jsError("selectBetweenHandles: " + err);
    }
};



